@@ -25,8 +25,8 @@ from rq import Worker, Connection
 from dallinger.config import get_config
 from dallinger.config import initialize_experiment_package
 from dallinger import data
-from dallinger.db import redis_conn
-from dallinger.deployment import _deploy_in_mode
+from dallinger import db
+from dallinger.deployment import deploy_sandbox_shared_setup
 from dallinger.deployment import DebugDeployment
 from dallinger.deployment import LoaderDeployment
 from dallinger.deployment import setup_experiment
@@ -62,24 +62,22 @@ header = r"""
 )
 
 
-def log(msg, delay=0.5, chevrons=True, verbose=True):
+def log(msg, chevrons=True, verbose=True):
     """Log a message to stdout."""
     if verbose:
         if chevrons:
             click.echo("\n❯❯ " + msg)
         else:
             click.echo(msg)
-        time.sleep(delay)
 
 
-def error(msg, delay=0.5, chevrons=True, verbose=True):
+def error(msg, chevrons=True, verbose=True):
     """Log a message to stdout."""
     if verbose:
         if chevrons:
             click.secho("\n❯❯ " + msg, err=True, fg="red")
         else:
             click.secho(msg, err=True, fg="red")
-        time.sleep(delay)
 
 
 class Output(object):
@@ -196,10 +194,7 @@ def verify_experiment_module(verbose):
     temp_package_name = "TEMP_VERIFICATION_PACKAGE"
     tmp = tempfile.mkdtemp()
     clone_dir = os.path.join(tmp, temp_package_name)
-    to_ignore = shutil.ignore_patterns(
-        os.path.join(".git", "*"), "*.db", "snapshots", "data", "server.log"
-    )
-    shutil.copytree(os.getcwd(), clone_dir, ignore=to_ignore)
+    ExperimentFileSource(os.getcwd()).selective_copy_to(clone_dir)
     initialize_experiment_package(clone_dir)
     from dallinger_experiment import experiment
 
@@ -216,22 +211,17 @@ def verify_experiment_module(verbose):
     if len(exps) == 0:
         log(
             "✗ experiment.py does not define an experiment class.",
-            delay=0,
             chevrons=False,
             verbose=verbose,
         )
         ok = False
     elif len(exps) == 1:
         log(
-            "✓ experiment.py defines 1 experiment",
-            delay=0,
-            chevrons=False,
-            verbose=verbose,
+            "✓ experiment.py defines 1 experiment", chevrons=False, verbose=verbose,
         )
     else:
         log(
             "✗ experiment.py defines more than one experiment class.",
-            delay=0,
             chevrons=False,
             verbose=verbose,
         )
@@ -254,7 +244,7 @@ def verify_config(verbose=True):
                 message = "Configuration for {} is invalid: ".format(config_key)
             else:
                 message = "Configuration is invalid: "
-            log("✗ " + message + str(e), delay=0, chevrons=False, verbose=verbose)
+            log("✗ " + message + str(e), chevrons=False, verbose=verbose)
 
             config_value = getattr(e, "dallinger_config_value", None)
             if verbose and config_value:
@@ -264,14 +254,13 @@ def verify_config(verbose=True):
     try:
         base_pay = config.get("base_payment")
     except KeyError:
-        log("✗ No value for base_pay.", delay=0, chevrons=False, verbose=verbose)
+        log("✗ No value for base_pay.", chevrons=False, verbose=verbose)
     else:
         dollarFormat = "{:.2f}".format(base_pay)
 
         if base_pay <= 0:
             log(
                 "✗ base_payment must be positive value in config.txt.",
-                delay=0,
                 chevrons=False,
                 verbose=verbose,
             )
@@ -281,7 +270,6 @@ def verify_config(verbose=True):
             log(
                 "✗ base_payment must be in [dollars].[cents] format in config.txt. Try changing "
                 "{0} to {1}.".format(base_pay, dollarFormat),
-                delay=0,
                 chevrons=False,
                 verbose=verbose,
             )
@@ -314,14 +302,13 @@ def verify_no_conflicts(verbose=True):
         if os.path.exists(f):
             log(
                 "✗ {} OVERWRITES shared frontend files inserted at run-time".format(f),
-                delay=0,
                 chevrons=False,
                 verbose=verbose,
             )
             conflicts = True
 
     if not conflicts:
-        log("✓ no file conflicts", delay=0, chevrons=False, verbose=verbose)
+        log("✓ no file conflicts", chevrons=False, verbose=verbose)
 
     return True
 
@@ -460,30 +447,59 @@ def _mturk_service_from_config(sandbox):
     )
 
 
+def prelaunch_db_bootstrapper(zip_path, log):
+    def bootstrap_db(heroku_app, config):
+        # Pre-populate the database if an archive was given
+        log("Ingesting dataset from {}...".format(os.path.basename(zip_path)))
+        engine = db.create_db_engine(heroku_app.db_url)
+        data.bootstrap_db_from_zip(zip_path, engine)
+
+    return bootstrap_db
+
+
+def _deploy_in_mode(mode, verbose, log, app=None, archive=None):
+    if app:
+        verify_id(None, None, app)
+
+    log(header, chevrons=False)
+    prelaunch = []
+    if archive:
+        archive_path = os.path.abspath(archive)
+        if not os.path.exists(archive_path):
+            raise click.BadParameter(
+                'Experiment archive "{}" does not exist.'.format(archive_path)
+            )
+        prelaunch.append(prelaunch_db_bootstrapper(archive_path, log))
+
+    config = get_config()
+    config.load()
+    config.extend({"mode": mode, "logfile": "-"})
+
+    deploy_sandbox_shared_setup(
+        log=log, verbose=verbose, app=app, prelaunch_actions=prelaunch
+    )
+
+
 @dallinger.command()
 @click.option("--verbose", is_flag=True, flag_value=True, help="Verbose mode")
 @click.option("--app", default=None, help="Experiment id")
+@click.option("--archive", default=None, help="Optional path to an experiment archive")
 @require_exp_directory
 @report_idle_after(21600)
-def sandbox(verbose, app):
+def sandbox(verbose, app, archive):
     """Deploy app using Heroku to the MTurk Sandbox."""
-    if app:
-        verify_id(None, None, app)
-    log(header, chevrons=False)
-    _deploy_in_mode("sandbox", app=app, verbose=verbose, log=log)
+    _deploy_in_mode(mode="sandbox", verbose=verbose, log=log, app=app, archive=archive)
 
 
 @dallinger.command()
 @click.option("--verbose", is_flag=True, flag_value=True, help="Verbose mode")
 @click.option("--app", default=None, help="ID of the deployed experiment")
+@click.option("--archive", default=None, help="Optional path to an experiment archive")
 @require_exp_directory
 @report_idle_after(21600)
-def deploy(verbose, app):
+def deploy(verbose, app, archive):
     """Deploy app using Heroku to MTurk."""
-    if app:
-        verify_id(None, None, app)
-    log(header, chevrons=False)
-    _deploy_in_mode("live", app=app, verbose=verbose, log=log)
+    _deploy_in_mode(mode="live", verbose=verbose, log=log, app=app, archive=archive)
 
 
 @dallinger.command()
@@ -541,8 +557,8 @@ def email_test():
     config = get_config()
     config.load()
     settings = EmailConfig(config)
-    out.log("Email Config", delay=0)
-    out.log(tabulate.tabulate(settings.as_dict().items()), chevrons=False, delay=0)
+    out.log("Email Config")
+    out.log(tabulate.tabulate(settings.as_dict().items()), chevrons=False)
     problems = settings.validate()
     if problems:
         out.error(
@@ -609,16 +625,15 @@ def compensate(recruiter, worker_id, email, dollars, sandbox):
                 "Compensation failed. The recruiter reports the following error:\n{}".format(
                     ex
                 ),
-                delay=0,
             )
             return
 
-    out.log("HIT Details", delay=0)
-    out.log(tabulate.tabulate(result["hit"].items()), chevrons=False, delay=0)
-    out.log("Qualification Details", delay=0)
-    out.log(tabulate.tabulate(result["qualification"].items()), chevrons=False, delay=0)
-    out.log("Worker Notification", delay=0)
-    out.log(tabulate.tabulate(result["email"].items()), chevrons=False, delay=0)
+    out.log("HIT Details")
+    out.log(tabulate.tabulate(result["hit"].items()), chevrons=False)
+    out.log("Qualification Details")
+    out.log(tabulate.tabulate(result["qualification"].items()), chevrons=False)
+    out.log("Worker Notification")
+    out.log(tabulate.tabulate(result["email"].items()), chevrons=False)
 
 
 @dallinger.command()
@@ -794,14 +809,11 @@ def extend_mturk_hit(hit_id, assignments, duration_hours, sandbox):
                 hit_id=hit_id, number=assignments, duration_hours=duration_hours
             )
         except MTurkServiceException as ex:
-            out.error(
-                "HIT extension failed with the following error:\n{}".format(ex),
-                delay=0,
-            )
+            out.error("HIT extension failed with the following error:\n{}".format(ex),)
             return
 
-    out.log("Updated HIT Details", delay=0)
-    out.log(tabulate.tabulate(hit_info.items()), chevrons=False, delay=0)
+    out.log("Updated HIT Details")
+    out.log(tabulate.tabulate(hit_info.items()), chevrons=False)
 
 
 @dallinger.command()
@@ -959,22 +971,20 @@ def verify():
     """Verify that app is compatible with Dallinger."""
     verbose = True
     log(
-        "Verifying current directory as a Dallinger experiment...",
-        delay=0,
-        verbose=verbose,
+        "Verifying current directory as a Dallinger experiment...", verbose=verbose,
     )
     ok = verify_package(verbose=verbose)
     if ok:
-        log("✓ Everything looks good!", delay=0, verbose=verbose)
+        log("✓ Everything looks good!", verbose=verbose)
     else:
-        log("☹ Some problems were found.", delay=0, verbose=verbose)
+        log("☹ Some problems were found.", verbose=verbose)
 
 
 @dallinger.command()
 def rq_worker():
     """Start an rq worker in the context of dallinger."""
     setup_experiment(log)
-    with Connection(redis_conn):
+    with Connection(db.redis_conn):
         # right now we care about low queue for bots
         worker = Worker("low")
         worker.work()
