@@ -26,6 +26,7 @@ from sqlalchemy import create_engine
 from sqlalchemy import distinct
 from sqlalchemy import func
 from sqlalchemy.orm import sessionmaker, scoped_session
+from sqlalchemy.orm.exc import NoResultFound, MultipleResultsFound
 
 from dallinger import recruiters
 from dallinger.config import get_config, LOCAL_CONFIG
@@ -82,6 +83,11 @@ class Experiment(object):
     exp_config = None
     replay_path = "/"
 
+    #: Constructor for Participant objects. Callable returning an instance of
+    #: :attr:`~dallinger.models.Participant` or a sub-class. Used by
+    #: :func:`~dallinger.experiment.Experiment.create_participant`.
+    participant_constructor = Participant
+
     def __init__(self, session=None):
         """Create the experiment class. Sets the default value of attributes."""
 
@@ -303,6 +309,62 @@ class Experiment(object):
         """
         network.add_node(node)
 
+    def create_participant(
+        self,
+        worker_id,
+        hit_id,
+        assignment_id,
+        mode,
+        recruiter_name=None,
+        fingerprint_hash=None,
+    ):
+        """Creates and returns a new participant object. Uses
+        :attr:`~dallinger.experiment.Experiment.participant_constructor` as the
+        constructor.
+
+        :param worker_id: the recruiter Worker Id
+        :type worker_id: str
+        :param hit_id: the recruiter HIT Id
+        :type hit_id: str
+        :param assignment_id: the recruiter Assignment Id
+        :type assignment_id: str
+        :param mode: the application mode
+        :type mode: str
+        :param recruiter_name: the recruiter name
+        :type recruiter_name: str
+        :returns: A :attr:`~dallinger.models.Participant` instance
+        """
+        if not recruiter_name:
+            recruiter = self.recruiter
+            if recruiter:
+                recruiter_name = recruiter.nickname
+
+        participant = self.participant_constructor(
+            recruiter_id=recruiter_name,
+            worker_id=worker_id,
+            assignment_id=assignment_id,
+            hit_id=hit_id,
+            mode=mode,
+            fingerprint_hash=fingerprint_hash,
+        )
+        self.session.add(participant)
+        return participant
+
+    def load_participant(self, assignment_id):
+        """Returns a participant object looked up by assignment_id.
+
+        Intended to allow a user to resume a session in a running experiment.
+
+        :param assignment_id: the recruiter Assignment Id
+        :type assignment_id: str
+        :returns: A ``Participant`` instance or ``None`` if there is not a
+                  single matching participant.
+        """
+        try:
+            return Participant.query.filter_by(assignment_id=assignment_id).one()
+        except (NoResultFound, MultipleResultsFound):
+            return None
+
     def data_check(self, participant):
         """Check that the data are acceptable.
 
@@ -703,13 +765,6 @@ class Experiment(object):
             )
         )
 
-        # Active participants have more than 10 infos
-        active_participants = 0
-        for p in participants:
-            if len(p.infos(failed="all")) > 10:
-                active_participants += 1
-        stats["Participants"]["active"] = active_participants
-
         # Count up our networks by role
         network_roles = self.session.query(Network.role, func.count(Network.role))
         network_counts = network_roles.group_by(Network.role).all()
@@ -837,7 +892,7 @@ class Experiment(object):
         :param \**kw: arguments passed in from the request. The ``model_type`` parameter
                       takes a ``str`` or iterable and queries all objects of those types,
                       ordered by ``id``.
-        :returns: Returns a a ``dict`` with DataTablesJS data and configuration, filters using
+        :returns: Returns a ``dict`` with DataTablesJS data and configuration, filters using
                   arbitrary keyword arguments. Should contain ``data`` and ``columns`` keys
                   at least, with ``columns`` containing data for all fields on all returned
                   objects.
@@ -853,6 +908,10 @@ class Experiment(object):
             model = getattr(models, model_type, None)
             for obj in model.query.order_by(model.id).all():
                 data = obj.__json__()
+                # Add participant worker_id to data, we normally leave it out of
+                # JSON renderings
+                if model_type == "Participant":
+                    data["worker_id"] = obj.worker_id
                 rows.append(data)
                 for key in data:
                     if key not in found_columns:
@@ -870,6 +929,51 @@ class Experiment(object):
             "columns": columns,
         }
 
+    def dashboard_database_actions(self):
+        """Returns a sequence of custom actions for the database dashboard. Each action
+           must have a ``title`` and a ``name`` corresponding to a method on the
+           experiment class.
+
+           The named methods should take a single ``data`` argument
+           which will be a list of dicts representing the datatables rendering of
+           a Dallinger model object. The named methods should return a ``dict``
+           containing a ``"message"`` which will be displayed in the dashboard.
+
+           Returns a single action referencing the
+           :func:`~dallinger.experiment.Experiment.dashboard_fail`
+           method by default.
+        """
+        return [{"name": "dashboard_fail", "title": "Fail Selected"}]
+
+    def dashboard_fail(self, data):
+        """Marks matching non-failed items as failed. Items are looked up by
+        ``id`` and ``object_type`` (e.g. ``"Participant"``).
+
+        :param data: A list of dicts representing model items to be marked as failed.
+                     Each must have an ``id`` and an ``object_type``
+        :type object_type: list
+
+        :returns: Returns a ``dict`` with a ``"message"`` string indicating how
+                  many items were successfully marked as failed.
+        """
+        counts = {}
+        for entry in data:
+            obj_id = entry.get("id")
+            object_type = entry.get("object_type")
+            model = getattr(models, object_type, None)
+            if model is not None:
+                obj = self.session.query(model).get(int(obj_id))
+                if obj is not None and not obj.failed:
+                    obj.fail()
+                    counts[object_type] = counts.get(object_type, 0) + 1
+        if not counts:
+            return {"message": "No nodes found to fail"}
+        return {
+            "message": "Failed {}".format(
+                ", ".join("{} {}s".format(c, t) for t, c in sorted(counts.items()))
+            )
+        }
+
     @property
     def usable_replay_range(self):
         """The range of times that represent the active part of the experiment"""
