@@ -742,35 +742,28 @@ def create_participant(worker_id, hit_id, assignment_id, mode):
         + 1
     )
 
-    recruiter_name = request.args.get("recruiter", "undefined")
-    if not recruiter_name or recruiter_name == "undefined":
-        recruiter = recruiters.from_config(_config())
-        if recruiter:
-            recruiter_name = recruiter.nickname
+    recruiter_name = request.args.get("recruiter")
 
     # Create the new participant.
-    participant = models.Participant(
-        recruiter_id=recruiter_name,
-        worker_id=worker_id,
-        assignment_id=assignment_id,
-        hit_id=hit_id,
-        mode=mode,
-        fingerprint_hash=fingerprint_hash,
+    exp = Experiment(session)
+    participant = exp.create_participant(
+        worker_id, hit_id, assignment_id, mode, recruiter_name, fingerprint_hash
     )
 
-    exp = Experiment(session)
-
+    session.flush()
     overrecruited = exp.is_overrecruited(nonfailed_count)
     if overrecruited:
         participant.status = "overrecruited"
 
-    session.add(participant)
-    session.flush()  # Make sure we know the id for the new row
     result = {"participant": participant.__json__()}
 
     # Queue notification to others in waiting room
     if exp.quorum:
-        quorum = {"q": exp.quorum, "n": nonfailed_count, "overrecruited": overrecruited}
+        quorum = {
+            "q": exp.quorum,
+            "n": nonfailed_count,
+            "overrecruited": participant.status == "overrecruited",
+        }
         db.queue_message(WAITING_ROOM_CHANNEL, dumps(quorum))
         result["quorum"] = quorum
 
@@ -792,6 +785,27 @@ def get_participant(participant_id):
     return success_response(participant=ppt.__json__())
 
 
+@app.route("/participant", methods=["POST"])
+def load_participant():
+    """Get the participant with an assignment id provided in the request.
+    Delegates to :func:`~dallinger.experiments.Experiment.load_participant`.
+    """
+    assignment_id = request_parameter("assignment_id", optional=True)
+    if assignment_id is None:
+        return error_response(
+            error_type="/participant POST: no participant found", status=403
+        )
+    exp = Experiment(session)
+    ppt = exp.load_participant(assignment_id)
+    if ppt is None:
+        return error_response(
+            error_type="/participant POST: no participant found", status=403
+        )
+
+    # return the data
+    return success_response(participant=ppt.__json__())
+
+
 @app.route("/network/<network_id>", methods=["GET"])
 def get_network(network_id):
     """Get the network with the given id."""
